@@ -1,10 +1,9 @@
 package Hailo;
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 
 use 5.010;
 use autodie qw(open close);
 use Any::Moose;
-use Any::Moose 'X::Getopt';
 use Any::Moose 'X::Types::'.any_moose() => [qw/Int Str Bool HashRef/];
 BEGIN {
     return unless Any::Moose::moose_is_preferred();
@@ -23,145 +22,54 @@ use Module::Pluggable (
 use List::Util qw(first);
 use namespace::clean -except => [ qw(meta plugins) ];
 
-with any_moose('X::Getopt::Dashes');
-
-has help => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => 'h',
-    cmd_flag      => 'help',
-    isa           => Bool,
-    is            => 'ro',
-    default       => 0,
-    documentation => 'This help message',
-);
-
-has print_version => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => 'v',
-    cmd_flag      => 'version',
-    documentation => 'Print version and exit',
-    isa           => Bool,
-    is            => 'ro',
-);
-
-has save_on_exit => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => 'a',
-    cmd_flag      => 'autosave',
-    documentation => 'Save the brain on exit (on by default)',
-    isa           => Bool,
-    is            => 'ro',
-    default       => 1,
-);
-
-has print_progress => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => 'p',
-    cmd_flag      => 'progress',
-    documentation => 'Print import progress with Term::ProgressBar',
-    isa           => Bool,
-    is            => 'ro',
-    default       => sub { _is_interactive() }
-);
-
-has learn_str => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => "l",
-    cmd_flag      => "learn",
-    documentation => "Learn from STRING",
-    isa           => Str,
-    is            => "ro",
-);
-
-has learn_reply_str => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => "L",
-    cmd_flag      => "learn-reply",
-    documentation => "Learn from STRING and reply to it",
-    isa           => Str,
-    is            => "ro",
-);
-
-has train_file => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => "t",
-    cmd_flag      => "train",
-    documentation => "Learn from all the lines in FILE, use - for STDIN",
-    isa           => Str,
-    is            => "ro",
-);
-
-has reply_str => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => "r",
-    cmd_flag      => "reply",
-    documentation => "Reply to STRING",
-    isa           => Str,
-    is            => "ro",
+has brain => (
+    isa => Str,
+    is  => 'rw',
 );
 
 has order => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => "o",
-    cmd_flag      => "order",
-    documentation => "Markov order",
-    isa           => Int,
-    is            => "ro",
-    default       => 2,
+    isa     => Int,
+    is      => 'rw',
+    default => 2,
+);
+
+has save_on_exit => (
+    isa     => Bool,
+    is      => 'rw',
+    default => 1,
 );
 
 has brain_resource => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => "b",
-    cmd_flag      => "brain",
-    documentation => "Load/save brain to/from FILE",
+    documentation => "Alias for `brain' for backwards compatability",
     isa           => Str,
-    is            => "ro",
+    is            => 'rw',
+    trigger       => sub {
+        my ($self, $brain) = @_;
+        $self->brain($brain);
+    },
 );
-
-has print_stats => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => "s",
-    cmd_flag      => "stats",
-    documentation => "Print statistics about the brain",
-    isa           => Bool,
-    is            => "ro",
-);
-
+    
 # working classes
 has storage_class => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => "S",
-    cmd_flag      => "storage",
-    documentation => "Use storage CLASS",
     isa           => Str,
-    is            => "ro",
+    is            => "rw",
     default       => "SQLite",
 );
 
 has tokenizer_class => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => "T",
-    cmd_flag      => "tokenizer",
-    documentation => "Use tokenizer CLASS",
     isa           => Str,
-    is            => "ro",
+    is            => "rw",
     default       => "Words",
 );
 
 has ui_class => (
-    traits        => [qw(Getopt)],
-    cmd_aliases   => "u",
-    cmd_flag      => "ui",
-    documentation => "Use UI CLASS",
     isa           => Str,
-    is            => "ro",
+    is            => "rw",
     default       => "ReadLine",
 );
 
 # Object arguments
 has storage_args => (
-    traits        => [qw(Getopt)],
     documentation => "Arguments for the Storage class",
     isa           => HashRef,
     coerce        => 1,
@@ -170,7 +78,6 @@ has storage_args => (
 );
 
 has tokenizer_args => (
-    traits        => [qw(Getopt)],
     documentation => "Arguments for the Tokenizer class",
     isa           => HashRef,
     is            => "ro",
@@ -178,7 +85,6 @@ has tokenizer_args => (
 );
 
 has ui_args => (
-    traits        => [qw(Getopt)],
     documentation => "Arguments for the UI class",
     isa           => HashRef,
     is            => "ro",
@@ -186,38 +92,35 @@ has ui_args => (
 );
 
 # Working objects
-has _storage_obj => (
-    traits      => [qw(NoGetopt)],
+has _storage => (
     does        => 'Hailo::Role::Storage',
     lazy_build  => 1,
     is          => 'ro',
     init_arg    => undef,
 );
 
-has _tokenizer_obj => (
-    traits      => [qw(NoGetopt)],
+has _tokenizer => (
     does        => 'Hailo::Role::Tokenizer',
     lazy_build  => 1,
     is          => 'ro',
     init_arg    => undef,
 );
 
-has _ui_obj => (
-    traits      => [qw(NoGetopt)],
+has _ui => (
     does        => 'Hailo::Role::UI',
     lazy_build  => 1,
     is          => 'ro',
     init_arg    => undef,
 );
 
-sub _build__storage_obj {
+sub _build__storage {
     my ($self) = @_;
     my $obj = $self->_new_class(
         "Storage",
         $self->storage_class,
         {
-            (defined $self->brain_resource
-             ? (brain => $self->brain_resource)
+            (defined $self->brain
+             ? (brain => $self->brain)
              : ()),
             order           => $self->order,
             arguments       => $self->storage_args,
@@ -227,7 +130,7 @@ sub _build__storage_obj {
     return $obj;
 }
 
-sub _build__tokenizer_obj {
+sub _build__tokenizer {
     my ($self) = @_;
     my $obj = $self->_new_class(
         "Tokenizer",
@@ -240,7 +143,7 @@ sub _build__tokenizer_obj {
     return $obj;
 }
 
-sub _build__ui_obj {
+sub _build__ui {
     my ($self) = @_;
     my $obj = $self->_new_class(
         "UI",
@@ -277,174 +180,77 @@ sub _new_class {
     return $pkg->new(%$args);
 }
 
-# Check validity of options
-before run => sub {
-    my ($self) = @_;
-
-    if (not $self->_storage_obj->ready and
-        (defined $self->reply_str or
-         defined $self->train_file or
-         defined $self->learn_str or
-         defined $self->learn_reply_str)) {
-        # TODO: Make this spew out the --help reply just like hailo
-        # with invalid options does usually, but only if run via
-        # ->new_with_options
-        die "To reply/train/learn you must specify options to initialize your storage backend";
-    }
-
-    return;
-};
-
-sub run {
-    my ($self) = @_;
-
-    if ($self->print_version) {
-        # Munging strictness because we don't have a version from a
-        # Git checkout. Dist::Zilla provides it.
-        no strict 'vars';
-        my $version = $VERSION // 'dev-git';
-
-        say "hailo $version";
-        return;
-    }
-
-    if (_is_interactive() and
-        $self->_storage_obj->ready and
-        not defined $self->train_file and
-        not defined $self->learn_str and
-        not defined $self->learn_reply_str and
-        not defined $self->reply_str and
-        not defined $self->print_stats) {
-
-        $self->_ui_obj->run($self);
-    }
-
-    $self->train($self->train_file) if defined $self->train_file;
-    $self->learn($self->learn_str) if defined $self->learn_str;
-
-    if (defined $self->learn_reply_str) {
-        my $answer = $self->learn_reply($self->learn_reply_str);
-        say $answer // "I don't know enough to answer you yet.";
-    }
-
-    if (defined $self->reply_str) {
-        my $answer = $self->reply($self->reply_str);
-        say $answer // "I don't know enough to answer you yet.";
-    }
-
-    if ($self->print_stats) {
-        my ($tok, $ex, $prev, $next) = $self->stats();
-        my $order = $self->_storage_obj->order;
-        say "Tokens: $tok";
-        say "Expression length: $order tokens";
-        say "Expressions: $ex";
-        say "Links to preceding tokens: $prev";
-        say "Links to following tokens: $next";
-    }
-
-    return;
-}
-
 sub save {
-    my $self = shift;
-    $self->_storage_obj->save(@_);
+    my ($self) = @_;
+    $self->_storage->save(@_);
     return;
 }
 
 sub train {
     my ($self, $input) = @_;
-    my $storage = $self->_storage_obj;
-    $storage->start_training();
 
-    my $got_filename = ref $input eq '';
+    $self->_storage->start_training();
 
-    my $fh;
-    if (ref $input eq 'GLOB') {
-        $fh = $input;
-    }
-    elsif (defined $input and $input eq "-") {
-        die "You must provide STDIN along with --train=-" if _is_interactive(*STDIN);
-        $fh = *STDIN;
-    }
-    elsif ($got_filename) {
-        open $fh, '<:encoding(utf8)', $input;
-    }
-
-    if ($self->print_progress and ref $input ne 'ARRAY' ) {
-        $self->_train_progress($fh, $input);
-    }
-    elsif (ref $input eq 'ARRAY') {
-        for my $line (@$input) {
-            $self->_learn_one($line);
+    given ($input) {
+        # With STDIN
+        when (not ref and defined and $_ eq '-') {
+            die "You must provide STDIN when training from '-'" if $self->_is_interactive(*STDIN);
+            $self->_train_fh(*STDIN);
         }
-    }
-    else {
-        while (my $line = <$fh>) {
-            chomp $line;
-            $self->_learn_one($line);
+        # With a filehandle
+        when (ref eq 'GLOB') {
+            $self->_train_fh($input);
+        }
+        # With a file
+        when (not ref) {
+            open my $fh, '<:encoding(utf8)', $input;
+            $self->_train_fh($fh, $input);
+        }
+        # With an Array
+        when (ref eq 'ARRAY') {
+            $self->_learn_one($_) for @$input;
+        }
+        # With something naughty
+        default {
+            die "Unknown input: $input";
         }
     }
 
-    close $fh if $got_filename;
-    $storage->stop_training();
+    $self->_storage->stop_training();
+
     return;
 }
 
-before _train_progress => sub {
-    require Term::ProgressBar;
-    Term::ProgressBar->import(2.00);
-    require File::CountLines;
-    File::CountLines->import('count_lines');
-    require Time::HiRes;
-    Time::HiRes->import(qw(gettimeofday tv_interval));
-    return;
-};
-
-sub _train_progress {
+sub _train_fh {
     my ($self, $fh, $filename) = @_;
-    my $lines = count_lines($filename);
-    my $progress = Term::ProgressBar->new({
-        name => "training from $filename",
-        count => $lines,
-        remove => 1,
-        ETA => 'linear',
-    });
-    $progress->minor(0);
-    my $next_update = 0;
-    my $start_time = [gettimeofday()];
 
-    my $i = 1; while (my $line = <$fh>) {
+    while (my $line = <$fh>) {
         chomp $line;
         $self->_learn_one($line);
-        if ($i >= $next_update) {
-            $next_update = $progress->update($.);
-
-            # The default Term::ProgressBar estimate for next updates
-            # is way too concervative. With a ~200k line file we only
-            # update every ~2k lines which is 10 seconds or so.
-            $next_update = (($next_update-$i) / 10) + $i;
-        }
-    } continue { $i++ }
-
-    $progress->update($lines) if $lines >= $next_update;
-    my $elapsed = tv_interval($start_time);
-    say "Imported in $elapsed seconds";
-
-    return;
+    }
 }
 
 sub learn {
     my ($self, $input) = @_;
     my $inputs;
-    if (ref $input eq 'ARRAY') {
-        $inputs = $input;
-    }
-    else {
-        die "Cannot learn from undef input" unless defined $input;
-        $inputs = [$input];
+
+    given ($input) {
+        when (not defined) {
+            die "Cannot learn from undef input";
+        }
+        when (not ref) {
+            $inputs = [$input];            
+        }
+        # With an Array
+        when (ref eq 'ARRAY') {
+            $inputs = $input
+        }
+        default {
+            die "Unknown input: $input";
+        }
     }
 
-    my $storage = $self->_storage_obj;
+    my $storage = $self->_storage;
 
     $storage->start_learning();
     $self->_learn_one($_) for @$inputs;
@@ -454,10 +260,10 @@ sub learn {
 
 sub _learn_one {
     my ($self, $input) = @_;
-    my $storage = $self->_storage_obj;
+    my $storage = $self->_storage;
     my $order   = $storage->order;
 
-    my $tokens = $self->_tokenizer_obj->make_tokens($input);
+    my $tokens = $self->_tokenizer->make_tokens($input);
 
     # only learn from inputs which are long enough
     return if @$tokens < $order;
@@ -474,8 +280,8 @@ sub learn_reply {
 
 sub reply {
     my ($self, $input) = @_;
-    my $storage = $self->_storage_obj;
-    my $toke    = $self->_tokenizer_obj;
+    my $storage = $self->_storage;
+    my $toke    = $self->_tokenizer;
 
     my $reply;
     if (defined $input) {
@@ -492,13 +298,13 @@ sub reply {
 
 sub stats {
     my ($self) = @_;
-    my $storage = $self->_storage_obj;
-    return $storage->totals();
+
+    return $self->_storage->totals();
 }
 
 sub DEMOLISH {
     my ($self) = @_;
-    $self->save if $self->save_on_exit;
+    $self->save_on_exit if $self->{_storage} and $self->save;
     return;
 }
 
@@ -507,64 +313,6 @@ sub _is_interactive {
     return IO::Interactive::is_interactive();
 }
 
-# --i--do-not-exist
-sub _getopt_spec_exception { goto &_getopt_full_usage }
-
-# --help
-sub _getopt_full_usage {
-    my ($self, $usage, $plain_str) = @_;
-
-    # If called from _getopt_spec_exception we get "Unknown option: foo"
-    my $warning = ref $usage eq 'ARRAY' ? $usage->[0] : undef;
-
-    my ($use, $options) = do {
-        # $plain_str under _getopt_spec_exception
-        my $out = $plain_str // $usage->text;
-
-        # The default getopt order sucks, use reverse sort order
-        chomp(my @out = split /^/, $out);
-        my $opt = join "\n", sort { $b cmp $a } @out[1 .. $#out];
-        ($out[0], $opt);
-    };
-    my $synopsis = do {
-        require Pod::Usage;
-        my $out;
-        open my $fh, '>', \$out;
-
-        require FindBin;
-        require File::Spec;
-        no warnings 'once';
-
-        Pod::Usage::pod2usage(
-            -input => File::Spec->catfile($FindBin::Bin, $FindBin::Script),
-            -sections => 'SYNOPSIS',
-            -output   => $fh,
-            -exitval  => 'noexit',
-        );
-        close $fh;
-
-        $out =~ s/\n+$//s;
-        $out =~ s/^Usage:/examples:/;
-
-        $out;
-    };
-
-    # Unknown option provided
-    print $warning if $warning;
-
-    print <<"USAGE";
-$use
-$options
-\n\tNote: All input/output and files are assumed to be UTF-8 encoded.
-USAGE
-
-    # Don't spew the example output when something's wrong with the
-    # options. It won't all fit on small terminals
-    say "\n", $synopsis unless $warning;
-
-    exit 1;
-}
-
 __PACKAGE__->meta->make_immutable;
 
 =encoding utf8
@@ -596,8 +344,8 @@ command-line invocation.
     # Heavy-duty training interface. Backends may drop some safety
     # features like journals or synchronous IO to train faster using
     # this mode.
-    $hailo->learn("megahal.trn");
-    $hailo->learn($filehandle);
+    $hailo->train("megahal.trn");
+    $hailo->train($filehandle);
 
     # Make the brain babble
     say $hailo->reply("hello good sir.");
@@ -773,9 +521,9 @@ In short, learning is lossy so an accurate conversion is impossible.
 
 =head1 ATTRIBUTES
 
-=head2 C<brain_resource>
+=head2 C<brain>
 
-The name of the resource (file name, database name) to use as storage.
+The name of the brain (file name, database name) to use as storage.
 There is no default. Whether this gets used at all depends on the
 storage backend, currently only SQLite uses it.
 
@@ -850,14 +598,6 @@ be passed as-is to the backend.
 Takes no arguments. Returns the number of tokens, expressions, previous
 token links and next token links.
 
-=head1 PRIVATE METHODS
-
-=head2 C<run>
-
-Run Hailo in accordance with the the attributes that were passed to
-it, this method is called by the L<hailo> command-line utility and the
-Hailo test suite, it's behavior is subject to change.
-
 =head1 SUPPORT
 
 You can join the IRC channel I<#hailo> on FreeNode if you have questions.
