@@ -1,5 +1,5 @@
 package Hailo::UI::ReadLine;
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 use 5.010;
 use Any::Moose;
 BEGIN {
@@ -25,10 +25,10 @@ sub BUILD {
 
 sub run {
     my ($self, $hailo) = @_;
-    my $name = ref $hailo;
+    my $name = 'Hailo';
     my $term = Term::ReadLine->new($name);
 
-    while (defined (my $line = $term->readline(lc($name) . '> '))) {
+    while (defined (my $line = $term->readline($name . '> '))) {
         $line = decode('utf8', $line);
         my $answer = $hailo->learn_reply($line);
         say $answer // "I don't know enough to answer you yet.";
